'-------------------------------------------------------------------------------
' Filename   : RC5-TX-Attiny13-5.bas
' Purpose    : RC5 Remote Control Transmitter
' Author     : Ralf Beesner; based on Bascom- AppNote #105 by Ger langezaal
'-------------------------------------------------------------------------------
'
' RC5 Code is sent one time after pressing a key
' Then the controller goes to powerdown
'
' max. 4 keys are attatched to PB1 .... PB4  against Ground
'
' Key 1: PB1
' Key 2: PB2
' Key 3: PB3
' Key 4: PB4
'
' As Timer0 is used to generate the necessary 36 kHz Sub-Carrier, the IR LED
' must be attached to PB0
'
' IR Led:          +5V <---[A Led K]---[220 Ohm]---> PB.0
' also possible:     0V <---[K LED A]---[220 Ohm]---> PB.0
'
'-------------------------------------------------------------------------------
'
$regfile = "attiny13.dat"
$crystal = 1200000
$hwstack = 32                                               ' default use 32 for the hardware stack
$swstack = 10                                               ' default use 10 for the SW stack
$framesize = 12                                             ' default use 40 for the frame space

Dim Rc5_address As Byte
Dim Rc5_command As Byte
Dim Rc5_bit_nr As Byte
Dim Toggle_bit As Boolean

Declare Sub Logic_0
Declare Sub Logic_1

Acsr.acd = 0                                                ' switch off analog-comparator to save energy
Ddrb = &B0000_0000                                          ' all pins are input
Portb = &B0001_1110                                         ' Pullups used for PB1 to PB4

Pcmsk = &B0001_1110                                         ' enable PCINT for PB1 to PB4
Gimsk.5 = 1                                                 ' activate PCints
Sreg.7 = 1                                                  ' activate Interrupts globally


' Timer 0 generates 36 kHz Carrier

Config Timer0 = Timer , Prescale = 1 , Compare A = Toggle , Clear Timer = 1
' Ocr0a = 138                                               ' value for 10MHz crystal (Attiny 2313)
' Ocr0a = 66                                                ' value for 4.8 MHz RC Osc (Attiny13)
Ocr0a = 17                                                  ' value for 1.2 MHz RC Osc (Attiny13)


Toggle_bit = 0                                              ' may be used as 7th data bit

'---[ set RC-5 System Address ]-------------------------------------------------

'
' Rc5_address = 0                          ' TV = 0
' Rc5_address = 5                          ' VCR = 5
' Rc5_address = 20                         ' CD = 20
'
' unused Rc5_addresses (reserved for future use) that do not conflict with existing remote controls:
' 7, 11, 13, 15, 24, 25, 27 - 31

Rc5_address = 21                                            ' this is my DVB remote control


'---[ main program loop ]-------------------------------------------------------

Do
    Disable Interrupts

    Rc5_command = Pinb                                      ' get RC-5 command from Dip Switches
    Rc5_command = Not Rc5_command                           ' invert it
    Rc5_command = Rc5_command And &B0001_1110               ' Filter only used bits

'---[ transmit Address and Command as 14 bit RC-5 code ]------------------------

    Ddrb.0 = 1                                              ' 36 kHz Carrier on
    Logic_1                                                 ' first start bit
    Logic_1                                                 ' second start bit
    If Toggle_bit = 0 Then Logic_0 Else Logic_1             ' toggle bit

    For Rc5_bit_nr = 4 To 0 Step -1                         ' 5 bit address, msb first
      If Rc5_address.rc5_bit_nr = 0 Then Logic_0 Else Logic_1
    Next

    For Rc5_bit_nr = 5 To 0 Step -1                         ' 6 bit command, msb first
      If Rc5_command.rc5_bit_nr = 0 Then Logic_0 Else Logic_1
    Next

    Ddrb.0 = 0                                              ' 36 kHz Carrier off
'    Waitms 87                                               'frame gap delay

    Enable Interrupts
    Powerdown

Loop

End

'-------------------------------------------------------------------------------

Sub Logic_0
  Ddrb.0 = 1                                                ' 36 kHz Carrier on
  Waitus 883
  Ddrb.0 = 0
  Waitus 884
End Sub

Sub Logic_1
  Ddrb.0 = 0
  Waitus 883
  Ddrb.0 = 1
  Waitus 884
End Sub

'
' for more RC-5 Remote Control info:
' http://www.ustr.net/infrared/infrared1.shtml
'
'-------------------------------------------------------------------------------